/*
 * COMMENT
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

public class A3JPanel extends JPanel implements KeyListener, ActionListener,
		MouseListener {
	public static final Color BACKGROUND_COLOUR = Color.GREEN;
	public static final Color LAUNCH_PAD_COLOUR = new Color(220, 66, 255);
	public static final Rectangle ABOVE_GAME_AREA = A3Constants.ABOVE_GAME_AREA;
	public static final Rectangle BELOW_GAME_AREA = A3Constants.BELOW_GAME_AREA;
	public static final Rectangle LEFT_OF_GAME_AREA = A3Constants.LEFT_OF_GAME_AREA;
	public static final Rectangle RIGHT_OF_GAME_AREA = A3Constants.RIGHT_OF_GAME_AREA;

	public static final Rectangle GAME_AREA = A3Constants.GAME_AREA;
	public static final Rectangle LAUNCH_PAD_AREA = A3Constants.LAUNCH_PAD_AREA;
	public static final Point CENTRE_POINT = A3Constants.CENTRE_POINT;
	public static final Point INFORMATION_POSITION1 = A3Constants.INFORMATION_POSITION1;

	public static final int[] MONEY_AMOUNTS = A3Constants.MONEY_AMOUNTS;

	public static final int UP = A3Constants.UP;
	public static final int DOWN = A3Constants.DOWN;
	public static final int LEFT = A3Constants.LEFT;
	public static final int RIGHT = A3Constants.RIGHT;

	private BlockOfCells blockOfCells = new BlockOfCells();
	private Hero hero = new Hero();
	private Timer t;
	private BandOfVillains bandOfVillains = new BandOfVillains();
	private static int sumMoney;
	private int timerCounter;
	private boolean gameHasEnded;
	private boolean showingTitleScreen;

	// only for test
	// private int a = 0;

	public A3JPanel() {
		setBackground(BACKGROUND_COLOUR);
		reset();
		timerCounter = 0;
		addKeyListener(this);
		addMouseListener(this);
		t = new Timer(100, this);
	}

	// -------------------------------------------------------
	// Initialise everything, ready for a new game.
	// -------------------------------------------------------
	private void reset() {
		timerCounter = 0;
		gameHasEnded = false;
		sumMoney = 0;
		showingTitleScreen = true;
	}

	// -------------------------------------------------------
	// Handle KeyEvents
	// -------------------------------------------------------
	@Override
	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub
		if (!showingTitleScreen) {
			switch (arg0.getKeyCode()) {
			case KeyEvent.VK_UP:
				// System.out.println("UP KEY Has Been Pressed");
				hero.setCurrentDirection(UP);
				break;
			case KeyEvent.VK_DOWN:
				hero.setCurrentDirection(DOWN);
				break;
			case KeyEvent.VK_LEFT:
				hero.setCurrentDirection(LEFT);
				break;
			case KeyEvent.VK_RIGHT:
				hero.setCurrentDirection(RIGHT);
				break;
			}
		} else {
			if (arg0.getKeyCode() == KeyEvent.VK_ENTER ){
				showingTitleScreen = false;
			}
		}
		System.out.println("---------------key pressed!");
		repaint();
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
	}

	// -------------------------------------------------------
	// Handle ActionEvents
	// -------------------------------------------------------
	@Override
	public void actionPerformed(ActionEvent arg0) {
		timerCounter++;
		if (timerCounter > 2) {
			hero.move();
			timerCounter = 0;
		}
		if (Math.random() > 0.9) {
			bandOfVillains.addAVillain();
		}
		sumMoney += blockOfCells.check007PositionAndGetMoney(new Point(
				(int) (hero.getArea().getX() + 0.5 * A3Constants.CELL_SIZE),
				(int) (hero.getArea().getY() + 0.5 * A3Constants.CELL_SIZE)));
		bandOfVillains.moveVillains();
		if (bandOfVillains.enemyHitsHero(hero)) {
			t.stop();
			gameHasEnded = true;
		}
		repaint();
	}

	// -------------------------------------------------------
	// Handle MouseEvents
	// -------------------------------------------------------
	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		System.out.println("mousePressed");
		if (!showingTitleScreen && !gameHasEnded && !t.isRunning()) {
			t.start();
		}
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	// -------------------------------------------------------
	// Draw everything
	// -------------------------------------------------------
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		// System.out.println("inside of paintComponent()");
		if (!showingTitleScreen) {
			drawCellBlock(g);
			hero.draw(g);
			bandOfVillains.drawVillains(g);
			g.drawString("You have $" + sumMoney, INFORMATION_POSITION1.x,
					INFORMATION_POSITION1.y);
		} else {
			g.drawString("Press Enter to Game", (int)GAME_AREA.getX() + 30, (int)GAME_AREA.getY() + 50);
		}
	}

	// -------------------------------------------------------
	// Drawing helper methods
	// -------------------------------------------------------
	private void drawCellBlock(Graphics g) {
		// System.out.println("inside of drewCellBlock()");
		blockOfCells.drawCells(g);

	}

	private void drawGameInformaton(Graphics g) {

	}
}
